@@ -1,4 +1,4 @@
-/* $Id: cut.c,v 1.19 2000/08/07 14:58:26 astyanax Exp $ */
+/* $Id: cut.c,v 1.20 2000/09/05 13:09:56 astyanax Exp $ */
 /**************************************************************************
  *   cut.c                                                                *
  *                                                                        *
@@ -152,7 +152,7 @@ int do_cut_text(void)
     }
 
     /* Must let cutbuffer get blown away first before we do this... */
-    if (fileptr == filebot)	
+    if (fileptr == filebot && !ISSET(MARK_ISSET))	
 	return 0;
 
 #ifndef NANO_SMALL
