@@ -1,4 +1,4 @@
-/* $Id: files.c,v 1.13 2000/08/06 21:13:45 astyanax Exp $ */
+/* $Id: files.c,v 1.14 2000/09/16 05:25:06 astyanax Exp $ */
 /**************************************************************************
  *   files.c                                                              *
  *                                                                        *
@@ -228,7 +228,8 @@ int open_file(char *filename, int insert, int quiet)
     } else {			/* File is A-OK */
 	if (S_ISDIR(fileinfo.st_mode)) {
 	    statusbar(_("File \"%s\" is a directory"), filename);
-	    new_file();
+	    if (!insert)
+		new_file();
 	    return -1;
 	}
 	if (!quiet)
