@@ -1,4 +1,4 @@
-/* $Id: global.c,v 1.6 2000/08/06 21:13:45 astyanax Exp $ */
+/* $Id: global.c,v 1.11 2000/09/11 22:33:54 astyanax Exp $ */
 /**************************************************************************
  *   global.c                                                             *
  *                                                                        *
@@ -39,7 +39,7 @@ int center_x = 0, center_y = 0;	/* Center of screen */
 WINDOW *edit;			/* The file portion of the editor  */
 WINDOW *topwin;			/* Top line of screen */
 WINDOW *bottomwin;		/* Bottom buffer */
-char *filename;			/* Name of the file */
+char filename[PATH_MAX];	/* Name of the file */
 int editwinrows = 0;		/* How many rows long is the edit
 				   window? */
 filestruct *current;		/* Current buffer pointer */
@@ -54,7 +54,7 @@ filestruct *editbot = NULL;	/* Same for the bottom */
 filestruct *filebot = NULL;	/* Last node in the file struct */
 filestruct *cutbuffer = NULL;	/* A place to store cut text */
 
-char *answer;			/* Answer str to many questions */
+char answer[132];		/* Answer str to many questions */
 int totlines = 0;		/* Total number of lines in the file */
 int totsize = 0;		/* Total number of bytes in the file */
 int placewewant = 0;		/* The collum we'd like the cursor
@@ -78,12 +78,15 @@ shortcut goto_list[GOTO_LIST_LEN];
 shortcut writefile_list[WRITEFILE_LIST_LEN];
 shortcut help_list[HELP_LIST_LEN];
 shortcut spell_list[SPELL_LIST_LEN];
+toggle toggles[TOGGLE_LEN];
 
 /* Regular expressions */
 
+#ifdef HAVE_REGEX_H
 regex_t search_regexp;          /* Global to store compiled search regexp */
 regmatch_t regmatches[10];      /* Match positions for parenthetical
                                    subexpressions, max of 10 */ 
+#endif
 
 /* Initialize a struct *without* our lovely braces =( */
 void sc_init_one(shortcut * s, int key, char *desc, char *help, int alt,
@@ -99,6 +102,56 @@ void sc_init_one(shortcut * s, int key, char *desc, char *help, int alt,
     s->func = func;
 }
 
+/* Initialize the toggles in the same manner */
+void toggle_init_one(toggle * t, int val, char *desc, int flag)
+{
+    t->val = val;
+    t->desc = desc;
+    t->flag = flag;
+}
+
+void toggle_init(void)
+{
+#ifndef NANO_SMALL
+    char *toggle_const_msg, *toggle_autoindent_msg, *toggle_suspend_msg,
+	*toggle_nohelp_msg, *toggle_picomode_msg, *toggle_mouse_msg,
+	*toggle_cuttoend_msg, *toggle_regexp_msg, *toggle_wrap_msg;
+
+    toggle_const_msg = _("Constant cursor position");
+    toggle_autoindent_msg = _("Auto indent");
+    toggle_suspend_msg = _("Suspend");
+    toggle_nohelp_msg = _("Help mode");
+    toggle_picomode_msg = _("Pico messages");
+    toggle_mouse_msg = _("Mouse support");
+    toggle_cuttoend_msg = _("Cut to end");
+#ifdef HAVE_REGEX_H
+    toggle_regexp_msg = _("Regular expressions");  
+#endif
+    toggle_wrap_msg = _("Auto wrap");
+
+    toggle_init_one(&toggles[0], TOGGLE_CONST_KEY, toggle_const_msg, 
+	CONSTUPDATE);
+    toggle_init_one(&toggles[1], TOGGLE_AUTOINDENT_KEY, toggle_autoindent_msg, 
+	AUTOINDENT);
+    toggle_init_one(&toggles[2], TOGGLE_SUSPEND_KEY, toggle_suspend_msg, 
+	SUSPEND);
+    toggle_init_one(&toggles[3], TOGGLE_NOHELP_KEY, toggle_nohelp_msg, 
+	NO_HELP);
+    toggle_init_one(&toggles[4], TOGGLE_PICOMODE_KEY, toggle_picomode_msg, 
+	PICO_MSGS);
+    toggle_init_one(&toggles[5], TOGGLE_WRAP_KEY, toggle_wrap_msg, 
+	NO_WRAP);
+    toggle_init_one(&toggles[6], TOGGLE_MOUSE_KEY, toggle_mouse_msg, 
+	USE_MOUSE);
+    toggle_init_one(&toggles[7], TOGGLE_CUTTOEND_KEY, toggle_cuttoend_msg, 
+	CUT_TO_END);
+#ifdef HAVE_REGEX_H
+    toggle_init_one(&toggles[8], TOGGLE_REGEXP_KEY, toggle_regexp_msg, 
+	USE_REGEXP);
+#endif
+#endif
+}
+
 void shortcut_init(void)
 {
     char *nano_help_msg = "", *nano_writeout_msg = "", *nano_exit_msg = "",
@@ -150,10 +203,9 @@ void shortcut_init(void)
     nano_cancel_msg = _("Cancel the current function");
 #endif
 
-
     if (ISSET(PICO_MSGS))
 	sc_init_one(&main_list[0], NANO_HELP_KEY, _("Get Help"),
-		    nano_help_msg, 0, 0, 0, VIEW, do_help);
+		    nano_help_msg, 0, NANO_HELP_FKEY, 0, VIEW, do_help);
     else
 	sc_init_one(&main_list[0], NANO_WRITEOUT_KEY, _("WriteOut"),
 		    nano_writeout_msg,
@@ -173,7 +225,8 @@ void shortcut_init(void)
 
     if (ISSET(PICO_MSGS))
 	sc_init_one(&main_list[3], NANO_JUSTIFY_KEY, _("Justify"),
-		    nano_justify_msg, 0, 0, 0, NOVIEW, do_justify);
+		    nano_justify_msg, 0, NANO_JUSTIFY_FKEY, 0, 
+		    NOVIEW, do_justify);
     else
 	sc_init_one(&main_list[3], NANO_REPLACE_KEY, _("Replace"),
 		    nano_replace_msg,
@@ -251,7 +304,8 @@ void shortcut_init(void)
 		    NANO_ALT_R, NANO_REPLACE_FKEY, 0, NOVIEW, do_replace);
     else
 	sc_init_one(&main_list[23], NANO_JUSTIFY_KEY, _("Justify"),
-		    nano_justify_msg, 0, 0, 0, NOVIEW, do_justify);
+		    nano_justify_msg, 0, NANO_JUSTIFY_FKEY, 0, 
+		    NOVIEW, do_justify);
 
     sc_init_one(&main_list[24], NANO_ENTER_KEY, _("Enter"),
 		nano_enter_msg,
@@ -263,7 +317,7 @@ void shortcut_init(void)
 		    NANO_ALT_G, NANO_GOTO_FKEY, 0, VIEW, do_gotoline_void);
     else
 	sc_init_one(&main_list[25], NANO_HELP_KEY, _("Get Help"),
-		    nano_help_msg, 0, 0, 0, VIEW, do_help);
+		    nano_help_msg, 0, NANO_HELP_FKEY, 0, VIEW, do_help);
 
 
     sc_init_one(&whereis_list[0], NANO_FIRSTLINE_KEY, _("First Line"),
@@ -340,4 +394,5 @@ void shortcut_init(void)
     sc_init_one(&spell_list[1], NANO_CANCEL_KEY, _("Cancel"),
 		nano_cancel_msg, 0, 0, 0, VIEW, 0);
 
+    toggle_init();
 }
