@@ -1,4 +1,4 @@
-/* $Id: nano.c,v 1.45 2000/08/06 21:09:27 astyanax Exp $ */
+/* $Id: nano.c,v 1.64 2000/09/15 15:46:32 astyanax Exp $ */
 /**************************************************************************
  *   nano.c                                                               *
  *                                                                        *
@@ -59,13 +59,12 @@
 #endif
 
 /* Former globals, now static */
-char *last_search = "\0";		/* Last string we searched for */
-char *last_replace = "\0";		/* Last replacement string */
 int fill = 0;			/* Fill - where to wrap lines, basically */
 static char *alt_speller;	/* Alternative spell command */
 struct termios oldterm;		/* The user's original term settings */
 static char *help_text_init = "";
 				/* Initial message, not including shortcuts */
+static struct sigaction act;	/* For all out fun signal handlers */
 
 /* What we do when we're all set to exit */
 RETSIGTYPE finish(int sigage)
@@ -148,10 +147,6 @@ void global_init(void)
     for (i = 0; i <= COLS - 1; i++)
 	hblank[i] = ' ';
     hblank[i] = 0;
-    last_search = nmalloc(132);
-    last_replace = nmalloc(132);
-    answer = nmalloc(132);
-
 }
 
 void init_help_msg(void)
@@ -171,9 +166,11 @@ void init_help_msg(void)
 	  "The bottom two lines show the most commonly used shortcuts "
 	  "in the editor.\n\n "
 	  "The notation for shortcuts is as follows: Control-key "
-	  "sequences are notated with a caret (^) symbol.  Alt-key "
-	  "sequences are notated with an at (@) symbol.  The following "
-	  "keystrokes are available in the main editor window. "
+	  "sequences are notated with a caret (^) symbol and are entered "
+	  "with the Control (Ctrl) key.  Escape-key sequences are notated "
+	  "with the Meta (M) symbol and can be entered using either the "
+	  "Esc, Alt or Meta key depending on your keyboard setup.  The "
+	  "following keystrokes are available in the main editor window. "
 	  "Optional keys are shown in parentheses:\n\n");
 #endif
 
@@ -322,7 +319,7 @@ void usage(void)
     printf(_("Option		Long option		Meaning\n"));
     printf(_
 	   (" -T 		--tabsize=[num]		Set width of a tab to num\n"));
-#ifdef _POSIX_VERSION
+#ifdef HAVE_REGEX_H
     printf(_
          (" -R		--regexp		Use regular expressions for search\n"));
 #endif
@@ -819,18 +816,6 @@ void do_wrap(filestruct * inptr, char input_char)
 	free(temp->data);
 	free(temp);
 
-
-	/* The next line line may need to be wrapped as well. */
-	current_y = old_y + 1;
-	current_x = strlen(inptr->next->data);
-	while (current_x >= 0) {
-	    if (isspace(inptr->next->data[current_x])
-		&& (current_x < fill)) break;
-	    current_x--;
-	}
-	if (current_x >= 0)
-	    check_wrap(inptr->next, ' ');
-
 	current_x = old_x;
 	current_y = old_y;
     }
@@ -1155,6 +1140,7 @@ int do_exit(void)
 void do_mouse(void)
 {
     MEVENT mevent;
+    int foo = 0, tab_found = 0;
 
     if (getmouse(&mevent) == ERR)
 	return;
@@ -1197,6 +1183,23 @@ void do_mouse(void)
 	}
     }
     current_x = mevent.x;
+    placewewant = current_x;
+    while(foo < current_x) {
+	if(current->data[foo] == NANO_CONTROL_I) {
+	    current_x -= tabsize - (foo % tabsize);
+	    tab_found = 1;
+	} else if(current->data[foo] & 0x80)
+	    ;
+	else if(current->data[foo] < 32)
+	    current_x--;
+	foo++;
+    }
+    /* This is where tab_found comes in.  I can't figure out why,
+     * but without it any line with a tab will place the cursor
+     * one character behind.  Whatever, this fixes it. */
+    if(tab_found == 1)
+	current_x++;
+
     if (current_x > strlen(current->data))
 	current_x = strlen(current->data);
 
@@ -1214,6 +1217,29 @@ RETSIGTYPE handle_hup(int signal)
     finish(1);
 }
 
+/* What do we do when we catch the suspend signal */
+RETSIGTYPE do_suspend(int signal)
+{
+
+    act.sa_handler = SIG_DFL;
+    sigemptyset(&act.sa_mask);
+    sigaction(SIGTSTP, &act, NULL);
+
+    endwin();
+    fprintf(stderr,"\n\n\n\n\nUse \"fg\" to return to nano\n");
+    raise(SIGTSTP);
+}
+
+/* Restore the suspend handler when we come back into the prog */
+RETSIGTYPE do_cont(int signal)
+{
+
+    act.sa_handler = do_suspend;
+    sigemptyset(&act.sa_mask);
+    sigaction(SIGTSTP, &act, NULL);
+    initscr();
+    total_refresh();
+}
 
 void handle_sigwinch(int s)
 {
@@ -1284,6 +1310,71 @@ void handle_sigwinch(int s)
 #endif
 }
 
+void signal_init(void)
+{
+
+    /* Trap SIGINT and SIGQUIT  cuz we want them to do useful things. */
+    memset(&act, 0, sizeof(struct sigaction));
+    act.sa_handler = SIG_IGN;
+    sigaction(SIGINT, &act, NULL);
+
+    if (!ISSET(SUSPEND)) {
+	sigaction(SIGTSTP, &act, NULL);
+    }
+    else
+    {
+	act.sa_handler = do_suspend;
+	sigaction(SIGTSTP, &act, NULL);
+
+	act.sa_handler = do_cont;
+	sigaction (SIGCONT, &act, NULL);
+    }
+
+
+    /* Trap SIGHUP  cuz we want to write the file out. */
+    act.sa_handler = handle_hup;
+    sigaction(SIGHUP, &act, NULL);
+
+    act.sa_handler = handle_sigwinch;
+    sigaction(SIGWINCH, &act, NULL);
+
+}
+
+void window_init(void)
+{
+    editwinrows = LINES - 5 + no_help();
+
+    /* Setup up the main text window */
+    edit = newwin(editwinrows, COLS, 2, 0);
+
+    /* And the other windows */
+    topwin = newwin(2, COLS, 0, 0);
+    bottomwin = newwin(3 - no_help(), COLS, LINES - 3 + no_help(), 0);\
+
+}
+
+void mouse_init(void)
+{
+#ifndef NANO_SMALL
+#ifdef NCURSES_MOUSE_VERSION
+    if (ISSET(USE_MOUSE)) {
+	keypad(edit, TRUE);
+	keypad(bottomwin, TRUE);
+
+	mousemask(BUTTON1_RELEASED, NULL);
+	mouseinterval(50);
+
+    }
+    else {
+	mousemask(0, NULL);
+	keypad(edit, FALSE);
+	keypad(bottomwin, FALSE);
+    }
+#endif
+#endif
+
+}
+
 int do_tab(void)
 {
     do_char('\t');
@@ -1496,6 +1587,11 @@ void help_init(void)
 	if (main_list[i].help != NULL)
 	    allocsize += strlen(main_list[i].help) + 15;
 
+    /* And for the toggle list, we also allocate space for extra text. */
+    for (i = 0; i <= TOGGLE_LEN - 1; i++)
+	if (toggles[i].desc != NULL)
+	    allocsize += strlen(toggles[i].desc) + 30;
+
     allocsize += strlen(help_text_init);
 
     if (help_text != NULL)
@@ -1508,7 +1604,7 @@ void help_init(void)
     strcpy(help_text, help_text_init);
 
     /* Now add our shortcut info */
-    for (i = 0; i < MAIN_LIST_LEN; i++) {
+    for (i = 0; i < MAIN_LIST_LEN - 1; i++) {
 	sofar = snprintf(buf, BUFSIZ, "^%c	", main_list[i].val + 64);
 
 	if (main_list[i].misc1 > KEY_F0 && main_list[i].misc1 <= KEY_F(64))
@@ -1518,35 +1614,100 @@ void help_init(void)
 	    sofar += snprintf(&buf[sofar], BUFSIZ - sofar, "	");
 
 	if (main_list[i].altval > 0)
-	    sofar += snprintf(&buf[sofar], BUFSIZ - sofar, "(@%c)	",
+	    sofar += snprintf(&buf[sofar], BUFSIZ - sofar, "(M-%c)	",
 			     main_list[i].altval - 32);
 	else
 	    sofar += snprintf(&buf[sofar], BUFSIZ - sofar, "	");
 
+
 	if (main_list[i].help != NULL)
 	    snprintf(&buf[sofar], BUFSIZ - sofar, "%s", main_list[i].help);
 
+
+	strcat(help_text, buf);
+	strcat(help_text, "\n");
+    }
+
+    /* And the toggles... */
+    for (i = 0; i <= TOGGLE_LEN - 1; i++) {
+	sofar = snprintf(buf, BUFSIZ, 
+	"M-%c			", toggles[i].val - 32 );
+
+	if (toggles[i].desc != NULL)
+	    snprintf(&buf[sofar], BUFSIZ - sofar, _("%s enable/disable"), 
+		toggles[i].desc);
+
 	strcat(help_text, buf);
 	strcat(help_text, "\n");
     }
 
 }
 
+void do_toggle(int which)
+{
+#ifndef NANO_SMALL
+    char *enabled = _("enabled");
+    char *disabled = _("disabled");
+
+    if (ISSET(toggles[which].flag))
+	UNSET(toggles[which].flag);
+    else
+	SET(toggles[which].flag);
+
+    switch (toggles[which].val) {
+    case TOGGLE_PICOMODE_KEY:
+	shortcut_init();
+	display_main_list();
+	break;
+    case TOGGLE_SUSPEND_KEY:
+	signal_init();
+	break;
+    case TOGGLE_MOUSE_KEY:
+	mouse_init();
+	break;
+    case TOGGLE_NOHELP_KEY:
+	wclear(bottomwin);
+	wrefresh(bottomwin);
+	window_init();
+	edit_refresh();
+	display_main_list();
+	break;
+    }
+
+    if (!ISSET(toggles[which].flag)) {
+	if (toggles[which].val == TOGGLE_NOHELP_KEY ||
+	   toggles[which].val == TOGGLE_WRAP_KEY)
+	    statusbar("%s %s", toggles[which].desc, enabled);
+	else
+	    statusbar("%s %s", toggles[which].desc, disabled);
+    } else {
+	if (toggles[which].val == TOGGLE_NOHELP_KEY ||
+	   toggles[which].val == TOGGLE_WRAP_KEY)
+	    statusbar("%s %s", toggles[which].desc, disabled);
+	else
+	    statusbar("%s %s", toggles[which].desc, enabled);
+    }
+    SET(DISABLE_CURPOS);
+
+#else
+    nano_small_msg();
+#endif
+}
+
 int main(int argc, char *argv[])
 {
     int optchr;
     int kbinput;		/* Input from keyboard */
     long startline = 0;		/* Line to try and start at */
-    struct sigaction act;	/* For our lovely signals */
     int keyhandled = 0;		/* Have we handled the keystroke yet? */
-    int tmpkey = 0, i;
+    int i;
     char *argv0;
     struct termios term;
 
 #ifdef HAVE_GETOPT_LONG
     int option_index = 0;
     struct option long_options[] = {
-#ifdef _POSIX_VERSION
+#ifdef HAVE_REGEX_H
         {"regexp", 0, 0, 'R'},
 #endif
 	{"version", 0, 0, 'V'},
@@ -1594,7 +1755,7 @@ int main(int argc, char *argv[])
 		finish(1);
 	    }
 	    break;
-#ifdef _POSIX_VERSION
+#ifdef HAVE_REGEX_H
 	case 'R':
 	    SET(USE_REGEXP);
 	    break;
@@ -1668,9 +1829,6 @@ int main(int argc, char *argv[])
 	|| (!argv0 && strstr(argv[0], "pico")))
 	SET(PICO_MSGS);
 
-    filename = nmalloc(PATH_MAX);
-    strcpy(filename, "");
-
     /* See if there's a non-option in argv (first non-option is the
        filename, if +LINE is not given) */
     if (argc == 1 || argc <= optind)
@@ -1693,11 +1851,13 @@ int main(int argc, char *argv[])
     /* First back up the old settings so they can be restored, duh */
     tcgetattr(0, &oldterm);
 
+#ifdef _POSIX_VDISABLE
     term = oldterm;
     term.c_cc[VINTR] = _POSIX_VDISABLE;
     term.c_cc[VQUIT] = _POSIX_VDISABLE;
     term.c_lflag &= ~IEXTEN;
     tcsetattr(0, TCSANOW, &term);
+#endif
 
     /* now ncurses init stuff... */
     initscr();
@@ -1712,44 +1872,14 @@ int main(int argc, char *argv[])
     shortcut_init();
     init_help_msg();
     help_init();
-
-    /* Trap SIGINT and SIGQUIT  cuz we want them to do useful things. */
-    memset(&act, 0, sizeof(struct sigaction));
-    act.sa_handler = SIG_IGN;
-    sigaction(SIGINT, &act, NULL);
-    sigaction(SIGQUIT, &act, NULL);
-
-    if (!ISSET(SUSPEND))
-	sigaction(SIGTSTP, &act, NULL);
-
-    /* Trap SIGHUP  cuz we want to write the file out. */
-    act.sa_handler = handle_hup;
-    sigaction(SIGHUP, &act, NULL);
-
-    act.sa_handler = handle_sigwinch;
-    sigaction(SIGWINCH, &act, NULL);
+    signal_init();
 
 #ifdef DEBUG
     fprintf(stderr, _("Main: set up windows\n"));
 #endif
 
-    /* Setup up the main text window */
-    edit = newwin(editwinrows, COLS, 2, 0);
-    keypad(edit, TRUE);
-
-#ifndef NANO_SMALL
-#ifdef NCURSES_MOUSE_VERSION
-    if (ISSET(USE_MOUSE)) {
-	mousemask(BUTTON1_RELEASED, NULL);
-	mouseinterval(50);
-    }
-#endif
-#endif
-
-    /* And the other windows */
-    topwin = newwin(2, COLS, 0, 0);
-    bottomwin = newwin(3 - no_help(), COLS, LINES - 3 + no_help(), 0);
-    keypad(bottomwin, TRUE);
+    window_init();
+    mouse_init();
 
 #ifdef DEBUG
     fprintf(stderr, _("Main: bottom win\n"));
@@ -1776,12 +1906,107 @@ int main(int argc, char *argv[])
     reset_cursor();
 
     while (1) {
+
+#ifndef _POSIX_VDISABLE
+	/* We're going to have to do it the old way, i.e. on cygwin */
+	raw();
+#endif
+
 	kbinput = wgetch(edit);
 	if (kbinput == 27) {	/* Grab Alt-key stuff first */
 	    switch (kbinput = wgetch(edit)) {
+	    /* Alt-O, suddenly very important ;) */
+	    case 79:
+		kbinput = wgetch(edit);
+		if (kbinput <= 'S' && kbinput >= 'P')
+		     kbinput = KEY_F(kbinput  - 79);
+#ifdef DEBUG
+		else {
+		    	fprintf(stderr, _("I got Alt-O-%c! (%d)\n"),
+			kbinput, kbinput);
+			break;
+		}
+#endif
+		break;
 	    case 91:
 
 		switch (kbinput = wgetch(edit)) {
+		case '1': /* Alt-[-1-[0-5,7-9] = F1-F8 in X at least */
+		    kbinput = wgetch(edit);
+		    if (kbinput >= '1' && kbinput <= '5') {
+			    kbinput = KEY_F(kbinput - 48);
+			    wgetch(edit);
+		    }
+		    else if (kbinput >= '7' && kbinput <= '9') {
+			    kbinput = KEY_F(kbinput - 49);
+			    wgetch(edit);
+		    }
+		    else if (kbinput == 126)
+			    kbinput = KEY_HOME;
+
+#ifdef DEBUG
+		    else {
+		    	    fprintf(stderr, _("I got Alt-[-1-%c! (%d)\n"),
+			    kbinput, kbinput);
+			    break;
+		    }
+#endif
+
+		    break;
+		case '2': /* Alt-[-2-[0,1,3,4] = F9-F12 in many terms */
+		    kbinput = wgetch(edit);
+		    switch (kbinput) {
+			case '0':
+			    kbinput = KEY_F(9);
+			    wgetch(edit);
+			    break;
+			case '1':
+			    kbinput = KEY_F(10);
+			    wgetch(edit);
+			    break;
+			case '3':
+			    kbinput = KEY_F(11);
+			    wgetch(edit);
+			    break;
+			case '4':
+			    kbinput = KEY_F(12);
+			    wgetch(edit);
+			    break;			    
+			case 126:	/* Hack, make insert key do something 
+					   usefile, like insert file */
+			    do_insertfile();
+			    keyhandled = 1;
+			    break;
+#ifdef DEBUG
+			default:
+		    	    fprintf(stderr, _("I got Alt-[-2-%c! (%d)\n"),
+			    kbinput, kbinput);
+			    break;
+#endif
+
+		    }
+		    break;
+		case '3': /* Alt-[-3 = Delete? */
+		    kbinput = NANO_DELETE_KEY;
+		    wgetch(edit);
+		    break;
+		case '4': /* Alt-[-4 = End? */
+		    kbinput = NANO_END_KEY;
+		    wgetch(edit);
+		    break;
+		case '5': /* Alt-[-5 = Page Up */
+		    kbinput = KEY_PPAGE;
+		    wgetch(edit);
+		    break;
+		case '6': /* Alt-[-6 = Page Down */
+		    kbinput = KEY_NPAGE;
+		    wgetch(edit);
+		    break;
+		case '[': /* Alt-[-[-[A-E], F1-F5 in linux console */
+		    kbinput = wgetch(edit);
+		     if (kbinput >= 'A' && kbinput <= 'E')
+		 	kbinput = KEY_F(kbinput - 64);
+		    break;
 		case 'A':
 		    kbinput = KEY_UP;
 		    break;
@@ -1800,30 +2025,6 @@ int main(int argc, char *argv[])
 		case 'F':
 		    kbinput = KEY_END;
 		    break;
-		case 49:	/* X window F-keys */
-		    tmpkey = wgetch(edit);
-		    kbinput = KEY_F(tmpkey) - 48;
-		    wgetch(edit);	/* Junk character */
-		    break;
-		case 53:	/* page up */
-		    kbinput = KEY_PPAGE;
-		    if ((kbinput = wgetch(edit)) == 126)
-			kbinput = KEY_PPAGE;	/* Ignore extra tilde */
-		    else {	/* I guess this could happen ;-) */
-			ungetch(kbinput);
-			continue;
-		    }
-		    break;
-		case 54:	/* page down */
-		    kbinput = KEY_NPAGE;
-		    if ((kbinput = wgetch(edit)) == 126)
-			kbinput = KEY_NPAGE;	/* Same thing here */
-		    else {
-			ungetch(kbinput);
-			continue;
-		    }
-		    break;
-
 		default:
 #ifdef DEBUG
 		    fprintf(stderr, _("I got Alt-[-%c! (%d)\n"),
@@ -1838,9 +2039,19 @@ int main(int argc, char *argv[])
 		for (i = 0; i <= MAIN_LIST_LEN - 1; i++)
 		    if (kbinput == main_list[i].altval ||
 			kbinput == main_list[i].altval - 32) {
-			kbinput = main_list[i].val;
-			break;
+			    kbinput = main_list[i].val;
+			    break;
 		    }
+#ifndef NANO_SMALL
+		/* And for toggle switches */
+		for (i = 0; i <= TOGGLE_LEN - 1 && !keyhandled; i++)
+		    if (kbinput == toggles[i].val ||
+			kbinput == toggles[i].val - 32) {
+			    do_toggle(i);
+			    keyhandled = 1;
+			    break;
+		    }
+#endif
 #ifdef DEBUG
 		fprintf(stderr, _("I got Alt-%c! (%d)\n"), kbinput,
 			kbinput);
@@ -1850,7 +2061,7 @@ int main(int argc, char *argv[])
 	}
 	/* Look through the main shortcut list to see if we've hit a
 	   shortcut key */
-	for (i = 0; i < MAIN_LIST_LEN; i++) {
+	for (i = 0; i < MAIN_LIST_LEN && !keyhandled; i++) {
 	    if (kbinput == main_list[i].val ||
 		(main_list[i].misc1 && kbinput == main_list[i].misc1) ||
 		(main_list[i].misc2 && kbinput == main_list[i].misc2)) {
@@ -1861,6 +2072,19 @@ int main(int argc, char *argv[])
 		keyhandled = 1;
 	    }
 	}
+#ifndef _POSIX_VDISABLE
+	/* Since we're in raw mode, we have to catch ^Q and ^S */
+	if (kbinput == 17 || kbinput == 19)
+	    keyhandled = 1;
+
+	/* And catch ^Z by hand when triggered */
+	if (kbinput == 26) {
+	    if (ISSET(SUSPEND))
+		do_suspend(0);
+	    keyhandled = 1;
+	}
+#endif
+
 	/* Last gasp, stuff that's not in the main lists */
 	if (!keyhandled)
 	    switch (kbinput) {
@@ -1891,8 +2115,12 @@ int main(int argc, char *argv[])
 		}
 		do_char(kbinput);
 	    }
-	if (ISSET(CONSTUPDATE))
-	    do_cursorpos();
+	if (ISSET(CONSTUPDATE)) {
+	    if (ISSET(DISABLE_CURPOS))
+		UNSET(DISABLE_CURPOS);
+	    else
+		do_cursorpos();
+	}
 
 	reset_cursor();
 	wrefresh(edit);
