@@ -1,4 +1,4 @@
-/* $Id: proto.h,v 1.14 2000/08/06 21:13:45 astyanax Exp $ */
+/* $Id: proto.h,v 1.17 2000/09/06 13:39:17 astyanax Exp $ */
 /**************************************************************************
  *   proto.h                                                              *
  *                                                                        *
@@ -22,7 +22,11 @@
 /* Externs */
 
 #include <sys/stat.h>
+
+#ifdef HAVE_REGEX_H
 #include <regex.h>
+#endif
+
 #include "nano.h"
 
 extern int center_x, center_y, editwinrows;
@@ -33,7 +37,8 @@ extern int totsize, temp_opt;
 extern int fill, flags,tabsize;
 
 extern WINDOW *edit, *topwin, *bottomwin;
-extern char *filename, *answer, *last_search, *last_replace;
+extern char filename[PATH_MAX];
+extern char answer[132];
 extern char *hblank, *help_text;
 extern struct stat fileinfo;
 extern filestruct *current, *fileage, *edittop, *editbot, *filebot; 
@@ -43,9 +48,14 @@ extern shortcut main_list[MAIN_LIST_LEN], whereis_list[WHEREIS_LIST_LEN];
 extern shortcut replace_list[REPLACE_LIST_LEN], goto_list[GOTO_LIST_LEN];
 extern shortcut writefile_list[WRITEFILE_LIST_LEN], help_list[HELP_LIST_LEN];
 extern shortcut spell_list[SPELL_LIST_LEN];
+
+#ifdef HAVE_REGEX_H
 extern int use_regexp, regexp_compiled;
 extern regex_t search_regexp;
 extern regmatch_t regmatches[10];  
+#endif
+
+extern toggle toggles[TOGGLE_LEN];
 
 /* Programs we want available */
 
@@ -80,7 +90,7 @@ void blank_bottombars(void);
 void check_wrap(filestruct * inptr, char ch);
 void dump_buffer(filestruct * inptr);
 void align(char **strp);
-void edit_refresh(void);
+void edit_refresh(void), edit_refresh_clearok(void);
 void edit_update(filestruct * fileptr, int topmidbot);
 void update_cursor(void);
 void delete_node(filestruct * fileptr);
