@@ -1,4 +1,4 @@
-/* $Id: search.c,v 1.16 2000/08/06 21:13:45 astyanax Exp $ */
+/* $Id: search.c,v 1.22 2000/09/10 05:06:09 astyanax Exp $ */
 /**************************************************************************
  *   search.c                                                             *
  *                                                                        *
@@ -34,9 +34,12 @@
 #define _(string) (string)
 #endif
 
+static char last_search[132];		/* Last string we searched for */
+static char last_replace[132];		/* Last replacement string */
+
 /* Regular expression helper functions */
 
-#ifdef _POSIX_VERSION
+#ifdef HAVE_REGEX_H
 void regexp_init(const char *regexp)
 {
     regcomp(&search_regexp, regexp, ISSET(CASE_SENSITIVE) ? 0 : REG_ICASE);
@@ -92,13 +95,13 @@ int search_init(int replacing)
 	return -1;
     } else if (i == -2) {	/* Same string */
 	strncpy(answer, last_search, 132);
-#ifdef _POSIX_VERSION
+#ifdef HAVE_REGEX_H
         if (ISSET(USE_REGEXP))
             regexp_init(answer);
 #endif
     } else if (i == 0) {	/* They entered something new */
 	strncpy(last_search, answer, 132);
-#ifdef _POSIX_VERSION
+#ifdef HAVE_REGEX_H
         if (ISSET(USE_REGEXP))
             regexp_init(answer);
 #endif
@@ -159,22 +162,23 @@ filestruct *findnextstr(int quiet, filestruct * begin, char *needle)
 
 	if (past_editbot)
 	    edit_update(current, CENTER);
+	placewewant = xplustabs();
 	reset_cursor();
     } else {			/* We're at EOF, go back to the top, once */
 
 	fileptr = fileage;
 
-	while (fileptr != current && fileptr != begin &&
+	while (fileptr != begin->next &&
 	       (found = strstrwrapper(fileptr->data, needle)) == NULL)
 	    fileptr = fileptr->next;
 
-	if (fileptr == begin) {
+	if (fileptr == begin->next) {
 	    if (!quiet)
 		statusbar(_("\"%s\" not found"), needle);
 
 	    return NULL;
 	}
-	if (fileptr != current) {	/* We found something */
+	else {	/* We found something */
 	    current = fileptr;
 	    current_x = 0;
 	    for (tmp = fileptr->data; tmp != found; tmp++)
@@ -185,12 +189,7 @@ filestruct *findnextstr(int quiet, filestruct * begin, char *needle)
 
 	    if (!quiet)
 		statusbar(_("Search Wrapped"));
-	} else {		/* Nada */
-
-	    if (!quiet)
-		statusbar(_("\"%s\" not found"), needle);
-	    return NULL;
-	}
+	} 
     }
 
     return fileptr;
@@ -201,8 +200,10 @@ void search_abort(void)
     UNSET(KEEP_CUTBUFFER);
     display_main_list();
     wrefresh(bottomwin);
+    if (ISSET(MARK_ISSET)) 
+	edit_refresh_clearok();
 
-#ifdef _POSIX_VERSION
+#ifdef HAVE_REGEX_H
     if (ISSET(REGEXP_COMPILED))
         regexp_cleanup();
 #endif
@@ -251,7 +252,7 @@ void replace_abort(void)
     search_abort();
 }
 
-#ifdef _POSIX_VERSION
+#ifdef HAVE_REGEX_H
 int replace_regexp(char *string, int create_flag)
 {
     /* split personality here - if create_flag is null, just calculate
@@ -321,7 +322,7 @@ char *replace_line()
     int search_match_count;
 
     /* Calculate size of new line */
-#ifdef _POSIX_VERSION
+#ifdef HAVE_REGEX_H
     if (ISSET(USE_REGEXP)) {
         search_match_count = regmatches[0].rm_eo -
             regmatches[0].rm_so;
@@ -349,7 +350,7 @@ char *replace_line()
     /* Replacement Text */
     if (!ISSET(USE_REGEXP))
         strcat(copy, last_replace);
-#ifdef _POSIX_VERSION
+#ifdef HAVE_REGEX_H
     else
         (void)replace_regexp(copy + current_x, 1);
 #endif
